"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const async_enumerable_1 = require("@reactive-js/async-enumerable");
const http_1 = require("@reactive-js/http");
const observable_1 = require("@reactive-js/observable");
const capabilities_1 = require("./capabilities");
const httpResponseBody_1 = require("./httpResponseBody");
const option_1 = require("@reactive-js/option");
exports.sendHttpRequestUsingXHR = request => observable_1.createObservable(subscriber => {
    const xhr = new XMLHttpRequest();
    const xhrSupportsResponseType = "responseType" in xhr;
    const bodyEnumerator = async_enumerable_1.identity().enumerateAsync(subscriber, 1);
    const body = new httpResponseBody_1.HttpResponseBodyImpl(bodyEnumerator);
    body.add(subscriber);
    subscriber.add(() => xhr.abort()).add(body);
    xhr.onerror = () => {
        const cause = new Error("Network request failed");
        subscriber.dispose({ cause });
    };
    xhr.onreadystatechange = () => {
        var _a;
        if (xhr.readyState === 2) {
            const headersRaw = (_a = xhr.getAllResponseHeaders()) !== null && _a !== void 0 ? _a : "";
            const headers = http_1.parseHeaders(headersRaw);
            const response = http_1.parseHttpResponseFromHeaders(xhr.status, headers, body);
            subscriber.dispatch({
                type: 4,
                response,
            });
            const { contentInfo } = response;
            if (xhrSupportsResponseType && option_1.isSome(contentInfo)) {
                const { contentLength, contentType: { type, subtype, params }, } = contentInfo;
                const hasCharset = option_1.isSome(params["charset"]);
                const responseIsText = hasCharset ||
                    type === "text" ||
                    subtype.indexOf("json") >= 0 ||
                    subtype.indexOf("text") >= 0 ||
                    subtype.indexOf("xml") >= 0;
                xhr.responseType = responseIsText
                    ? "text"
                    : contentLength > 0 && capabilities_1.supportsArrayBuffer
                        ? "arraybuffer"
                        : capabilities_1.supportsBlob
                            ? "blob"
                            : "";
            }
        }
        else if (xhr.readyState === 4) {
            bodyEnumerator.dispatch(xhr.response);
        }
    };
    xhr.onloadstart = () => {
        subscriber.dispatch({
            type: 1,
        });
    };
    xhr.upload.onprogress = ev => {
        const { loaded: count } = ev;
        subscriber.dispatch({
            type: 2,
            count,
        });
    };
    xhr.upload.onload = _ => {
        subscriber.dispatch({
            type: 3,
        });
    };
    xhr.ontimeout = () => {
        const cause = new Error("Network request failed");
        subscriber.dispose({ cause });
    };
    if (request.credentials === "include") {
        xhr.withCredentials = true;
    }
    else if (request.credentials === "omit") {
        xhr.withCredentials = false;
    }
    xhr.open(request.method, request.uri.toString(), true);
    http_1.writeHttpRequestHeaders(request, (k, v) => xhr.setRequestHeader(k, v));
    xhr.send(request.body);
});
//# sourceMappingURL=xhr.js.map