import { __awaiter } from "tslib";
import { httpRequestToUntypedHeaders, parseHttpResponseFromHeaders, } from "@reactive-js/http";
import { createObservable, fromPromise, publish, } from "@reactive-js/observable";
import { supportsArrayBuffer, supportsBlob } from "./capabilities";
import { HttpResponseBodyImpl } from "./httpResponseBody";
import { isSome, none } from "@reactive-js/option";
import { pipe } from "@reactive-js/pipe";
export const sendHttpRequestUsingFetch = request => {
    const { cache, credentials, integrity, method, mode, redirect, referrerPolicy, uri, } = request;
    const url = uri.toString();
    const headers = httpRequestToUntypedHeaders(request);
    return createObservable((subscriber) => __awaiter(void 0, void 0, void 0, function* () {
        const abortController = new AbortController();
        subscriber.add(() => abortController.abort());
        subscriber.dispatch({
            type: 1,
        });
        try {
            const fetchResponse = yield fetch(url, {
                cache,
                credentials,
                headers,
                integrity,
                method,
                mode,
                redirect,
                referrerPolicy,
                signal: abortController.signal,
            });
            const responseHeaders = {};
            fetchResponse.headers.forEach((v, k) => {
                responseHeaders[k] = v;
            });
            const responseNoBody = parseHttpResponseFromHeaders(fetchResponse.status, responseHeaders, none);
            const loadBodyContent = () => __awaiter(void 0, void 0, void 0, function* () {
                const { contentInfo } = responseNoBody;
                if (isSome(contentInfo)) {
                    const { contentLength, contentType: { type, subtype, params }, } = contentInfo;
                    const hasCharset = isSome(params["charset"]);
                    const responseIsText = hasCharset ||
                        type === "text" ||
                        subtype.indexOf("json") >= 0 ||
                        subtype.indexOf("text") >= 0 ||
                        subtype.indexOf("xml") >= 0;
                    if (responseIsText) {
                        return fetchResponse.text();
                    }
                    else if (contentLength > 0 && supportsArrayBuffer) {
                        return fetchResponse.arrayBuffer();
                    }
                    else if (supportsBlob) {
                        return fetchResponse.blob();
                    }
                }
                return yield "";
            });
            const bodyObservable = pipe(fromPromise(loadBodyContent), publish(subscriber, 1));
            const body = new HttpResponseBodyImpl(bodyObservable);
            subscriber.add(body);
            body.add(subscriber);
            const response = Object.assign(Object.assign({}, responseNoBody), { body });
            subscriber.dispatch({
                type: 4,
                response,
            });
        }
        catch (cause) {
            subscriber.dispose({ cause });
        }
    }));
};
//# sourceMappingURL=fetch.js.map